@@ -62,7 +62,7 @@
 #include "clutter-debug.h"
 #include "clutter-device-manager-private.h"
 #include "clutter-enum-types.h"
-#include "clutter-id-pool.h"
+#include "clutter-event-private.h"
 #include "clutter-main.h"
 #include "clutter-marshal.h"
 #include "clutter-master-clock.h"
@@ -137,6 +137,13 @@ struct _ClutterStagePrivate
 
   GHashTable *devices;
 
+  GTimer *fps_timer;
+  gint32 timer_n_frames;
+
+#ifdef CLUTTER_ENABLE_DEBUG
+  gulong redraw_count;
+#endif /* CLUTTER_ENABLE_DEBUG */
+
   guint relayout_pending       : 1;
   guint redraw_pending         : 1;
   guint is_fullscreen          : 1;
@@ -150,6 +157,7 @@ struct _ClutterStagePrivate
   guint dirty_projection       : 1;
   guint have_valid_pick_buffer : 1;
   guint accept_focus           : 1;
+  guint motion_events_enabled  : 1;
 };
 
 enum
@@ -798,6 +806,52 @@ _clutter_stage_maybe_relayout (ClutterActor *actor)
     }
 }
 
+static void
+clutter_stage_do_redraw (ClutterStage *stage)
+{
+  ClutterBackend *backend = clutter_get_default_backend ();
+  ClutterActor *actor = CLUTTER_ACTOR (stage);
+  ClutterStagePrivate *priv = stage->priv;
+
+  CLUTTER_TIMESTAMP (SCHEDULER, "Redraw started for %s[%p]",
+                     _clutter_actor_get_debug_name (actor),
+                     stage);
+
+  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
+  _clutter_stage_reset_picks_per_frame_counter (stage);
+
+  _clutter_backend_ensure_context (backend, stage);
+
+  if (clutter_get_show_fps ())
+    {
+      if (priv->fps_timer == NULL)
+        priv->fps_timer = g_timer_new ();
+    }
+
+  _clutter_stage_maybe_setup_viewport (stage);
+
+  _clutter_backend_redraw (backend, stage);
+
+  if (clutter_get_show_fps ())
+    {
+      priv->timer_n_frames += 1;
+
+      if (g_timer_elapsed (priv->fps_timer, NULL) >= 1.0)
+        {
+          g_print ("*** FPS for %s: %i ***\n",
+                   _clutter_actor_get_debug_name (actor),
+                   priv->timer_n_frames);
+
+          priv->timer_n_frames = 0;
+          g_timer_start (priv->fps_timer);
+        }
+    }
+
+  CLUTTER_TIMESTAMP (SCHEDULER, "Redraw finished for %s[%p]",
+                     _clutter_actor_get_debug_name (actor),
+                     stage);
+}
+
 /**
  * _clutter_stage_do_update:
  * @stage: A #ClutterStage
@@ -829,19 +883,20 @@ _clutter_stage_do_update (ClutterStage *stage)
 
   _clutter_stage_maybe_finish_queue_redraws (stage);
 
-  CLUTTER_NOTE (PAINT, "redrawing via idle for stage[%p]", stage);
-  _clutter_do_redraw (stage);
+  clutter_stage_do_redraw (stage);
 
   /* reset the guard, so that new redraws are possible */
   priv->redraw_pending = FALSE;
 
-  if (CLUTTER_CONTEXT ()->redraw_count > 0)
+#ifdef CLUTTER_ENABLE_DEBUG
+  if (priv->redraw_count > 0)
     {
       CLUTTER_NOTE (SCHEDULER, "Queued %lu redraws during the last cycle",
-                    CLUTTER_CONTEXT ()->redraw_count);
+                    priv->redraw_count);
 
-      CLUTTER_CONTEXT ()->redraw_count = 0;
+      priv->redraw_count = 0;
     }
+#endif /* CLUTTER_ENABLE_DEBUG */
 
   return TRUE;
 }
@@ -1199,6 +1254,9 @@ clutter_stage_finalize (GObject *object)
 
   g_hash_table_destroy (priv->devices);
 
+  if (priv->fps_timer != NULL)
+    g_timer_destroy (priv->fps_timer);
+
   G_OBJECT_CLASS (clutter_stage_parent_class)->finalize (object);
 }
 
@@ -1579,6 +1637,7 @@ clutter_stage_init (ClutterStage *self)
   priv->use_fog                = FALSE;
   priv->throttle_motion_events = TRUE;
   priv->min_size_changed       = FALSE;
+  priv->motion_events_enabled  = clutter_get_motion_events_enabled ();
 
   priv->color = default_stage_color;
 
@@ -1685,7 +1744,7 @@ clutter_stage_set_color (ClutterStage       *stage,
 /**
  * clutter_stage_get_color:
  * @stage: A #ClutterStage
- * @color: (out): return location for a #ClutterColor
+ * @color: (out caller-allocates): return location for a #ClutterColor
  *
  * Retrieves the stage color.
  */
@@ -3189,20 +3248,26 @@ _clutter_stage_queue_actor_redraw (ClutterStage *stage,
 {
   ClutterStagePrivate *priv = stage->priv;
 
-  CLUTTER_NOTE (PAINT, "Redraw request number %lu",
-                CLUTTER_CONTEXT ()->redraw_count + 1);
-
   if (!priv->redraw_pending)
     {
       ClutterMasterClock *master_clock;
 
+      CLUTTER_NOTE (PAINT, "First redraw request");
+
       priv->redraw_pending = TRUE;
 
       master_clock = _clutter_master_clock_get_default ();
       _clutter_master_clock_start_running (master_clock);
     }
+#ifdef CLUTTER_ENABLE_DEBUG
   else
-    CLUTTER_CONTEXT ()->redraw_count += 1;
+    {
+      CLUTTER_NOTE (PAINT, "Redraw request number %lu",
+                    priv->redraw_count + 1);
+
+      priv->redraw_count += 1;
+    }
+#endif /* CLUTTER_ENABLE_DEBUG */
 
   /* We have an optimization in _clutter_do_pick to detect when the
    * scene is static so we can cache a full, un-clipped pick buffer to
@@ -3402,3 +3467,23 @@ _clutter_stage_has_device (ClutterStage       *stage,
 
   return g_hash_table_lookup (priv->devices, device) != NULL;
 }
+
+void
+_clutter_stage_set_motion_events_enabled (ClutterStage *stage,
+                                          gboolean      enabled)
+{
+  ClutterStagePrivate *priv = stage->priv;
+
+  enabled = !!enabled;
+
+  if (priv->motion_events_enabled != enabled)
+    {
+      priv->motion_events_enabled = enabled;
+    }
+}
+
+gboolean
+_clutter_stage_get_motion_events_enabled (ClutterStage *stage)
+{
+  return stage->priv->motion_events_enabled;
+}
