@@ -45,17 +45,18 @@
 #endif
 
 #include "clutter-texture.h"
+
+#include "clutter-actor-private.h"
+#include "clutter-debug.h"
+#include "clutter-enum-types.h"
+#include "clutter-feature.h"
 #include "clutter-main.h"
 #include "clutter-marshal.h"
-#include "clutter-feature.h"
-#include "clutter-util.h"
 #include "clutter-private.h"
 #include "clutter-scriptable.h"
-#include "clutter-debug.h"
-#include "clutter-fixed.h"
-#include "clutter-enum-types.h"
-
-#include "cogl/cogl.h"
+#include "clutter-shader.h"
+#include "clutter-stage-private.h"
+#include "clutter-util.h"
 
 static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
 
@@ -413,39 +414,53 @@ set_viewport_with_buffer_under_fbo_source (ClutterActor *fbo_source,
                                            int viewport_width,
                                            int viewport_height)
 {
-  ClutterVertex verts[4];
-  float x_min = G_MAXFLOAT, y_min = G_MAXFLOAT;
-  int x_offset, y_offset;
-  int i;
+  ClutterActorBox box = { 0, };
+  float x_offset, y_offset;
 
-  /* Get the actors allocation transformed into screen coordinates.
-   *
-   * XXX: Note: this may not be a bounding box for the actor, since an
-   * actor with depth may escape the box due to its perspective
-   * projection. */
-  clutter_actor_get_abs_allocation_vertices (fbo_source, verts);
-
-  for (i = 0; i < G_N_ELEMENTS (verts); ++i)
+  if (clutter_actor_get_paint_box (fbo_source, &box))
+    clutter_actor_box_get_origin (&box, &x_offset, &y_offset);
+  else
     {
-      if (verts[i].x < x_min)
-	x_min = verts[i].x;
-      if (verts[i].y < y_min)
-	y_min = verts[i].y;
-    }
+      /* As a fallback when the paint box can't be determined we use
+       * the transformed allocation to come up with an offset instead.
+       *
+       * FIXME: when we don't have a paint box we should instead be
+       * falling back to a stage sized fbo with an offset of (0,0)
+       */
 
-  /* XXX: It's not good enough to round by simply truncating the fraction here
-   * via a cast, as it results in offscreen rendering being offset by 1 pixel
-   * in many cases... */
+      ClutterVertex verts[4];
+      float x_min = G_MAXFLOAT, y_min = G_MAXFLOAT;
+      int i;
+
+      /* Get the actors allocation transformed into screen coordinates.
+       *
+       * XXX: Note: this may not be a bounding box for the actor, since an
+       * actor with depth may escape the box due to its perspective
+       * projection. */
+      clutter_actor_get_abs_allocation_vertices (fbo_source, verts);
+
+      for (i = 0; i < G_N_ELEMENTS (verts); ++i)
+        {
+          if (verts[i].x < x_min)
+           x_min = verts[i].x;
+          if (verts[i].y < y_min)
+           y_min = verts[i].y;
+        }
+
+      /* XXX: It's not good enough to round by simply truncating the fraction here
+       * via a cast, as it results in offscreen rendering being offset by 1 pixel
+       * in many cases... */
 #define ROUND(x) ((x) >= 0 ? (long)((x) + 0.5) : (long)((x) - 0.5))
 
-  x_offset = ROUND (-x_min);
-  y_offset = ROUND (-y_min);
+      x_offset = ROUND (x_min);
+      y_offset = ROUND (y_min);
 
 #undef ROUND
+    }
 
   /* translate the viewport so that the source actor lands on the
    * sub-region backed by the offscreen framebuffer... */
-  cogl_set_viewport (x_offset, y_offset, viewport_width, viewport_height);
+  cogl_set_viewport (-x_offset, -y_offset, viewport_width, viewport_height);
 }
 
 static void
@@ -453,27 +468,26 @@ update_fbo (ClutterActor *self)
 {
   ClutterTexture        *texture = CLUTTER_TEXTURE (self);
   ClutterTexturePrivate *priv = texture->priv;
-  ClutterMainContext    *context;
+  ClutterActor          *head;
   ClutterShader         *shader = NULL;
   ClutterActor          *stage = NULL;
-  ClutterPerspective     perspective;
+  CoglMatrix             projection;
   CoglColor              transparent_col;
 
-  context = _clutter_context_get_default ();
+  head = _clutter_context_peek_shader_stack ();
+  if (head != NULL)
+    shader = clutter_actor_get_shader (head);
 
-  if (context->shaders)
-    shader = clutter_actor_get_shader (context->shaders->data);
-
-  /* Temporarily turn off the shader on the top of the context's shader stack,
-   * to restore the GL pipeline to it's natural state.
+  /* Temporarily turn off the shader on the top of the context's
+   * shader stack, to restore the GL pipeline to it's natural state.
    */
-  if (shader)
+  if (shader != NULL)
     clutter_shader_set_is_enabled (shader, FALSE);
 
   /* Redirect drawing to the fbo */
   cogl_push_framebuffer (priv->fbo_handle);
 
-  if ((stage = clutter_actor_get_stage (self)))
+  if ((stage = clutter_actor_get_stage (self)) != NULL)
     {
       gfloat stage_width, stage_height;
       ClutterActor *source_parent;
@@ -490,24 +504,21 @@ update_fbo (ClutterActor *self)
        * under the actor.
        */
 
-      clutter_stage_get_perspective (CLUTTER_STAGE (stage), &perspective);
+      _clutter_stage_get_projection_matrix (CLUTTER_STAGE (stage), &projection);
+
+      /* Set the projection matrix modelview matrix as it is for the
+       * stage... */
+      cogl_set_projection_matrix (&projection);
+
       clutter_actor_get_size (stage, &stage_width, &stage_height);
 
-      /* Set the projection matrix modelview matrix and viewport size as
-       * they are for the stage... */
-      _cogl_setup_viewport (stage_width, stage_height,
-                            perspective.fovy,
-                            perspective.aspect,
-                            perspective.z_near,
-                            perspective.z_far);
-
-      /* Negatively offset the viewport so that the offscreen framebuffer is
-       * position underneath the fbo_source actor... */
+      /* Set a negatively offset the viewport so that the offscreen
+       * framebuffer is position underneath the fbo_source actor... */
       set_viewport_with_buffer_under_fbo_source (priv->fbo_source,
                                                  stage_width,
                                                  stage_height);
 
-      /* Reapply the source's parent transformations */
+      /* Apply the source's parent transformations to the modelview */
       if ((source_parent = clutter_actor_get_parent (priv->fbo_source)))
         {
           CoglMatrix modelview;
@@ -542,7 +553,7 @@ update_fbo (ClutterActor *self)
   cogl_pop_framebuffer ();
 
   /* If there is a shader on top of the shader stack, turn it back on. */
-  if (shader)
+  if (shader != NULL)
     clutter_shader_set_is_enabled (shader, TRUE);
 }
 
@@ -674,6 +685,15 @@ clutter_texture_paint (ClutterActor *self)
   gen_texcoords_and_draw_cogl_rectangle (self);
 }
 
+static gboolean
+clutter_texture_get_paint_volume (ClutterActor       *self,
+                                  ClutterPaintVolume *volume)
+{
+  return _clutter_actor_set_default_paint_volume (self,
+                                                  CLUTTER_TYPE_TEXTURE,
+                                                  volume);
+}
+
 static void
 clutter_texture_async_data_free (ClutterTextureAsyncData *data)
 {
@@ -945,10 +965,11 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   g_type_class_add_private (klass, sizeof (ClutterTexturePrivate));
 
-  actor_class->paint          = clutter_texture_paint;
-  actor_class->pick           = clutter_texture_pick;
-  actor_class->realize        = clutter_texture_realize;
-  actor_class->unrealize      = clutter_texture_unrealize;
+  actor_class->paint            = clutter_texture_paint;
+  actor_class->pick             = clutter_texture_pick;
+  actor_class->get_paint_volume = clutter_texture_get_paint_volume;
+  actor_class->realize          = clutter_texture_realize;
+  actor_class->unrealize        = clutter_texture_unrealize;
 
   actor_class->get_preferred_width  = clutter_texture_get_preferred_width;
   actor_class->get_preferred_height = clutter_texture_get_preferred_height;
@@ -969,7 +990,8 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_boolean ("disable-slicing",
                                 P_("Disable Slicing"),
-                                P_("Forces the underlying texture to be singlular and not made of of smaller space saving inidivual textures."),
+                                P_("Forces the underlying texture to be singular and not made of smaller space saving "
+                                   "individual textures"),
                                 FALSE,
                                 G_PARAM_CONSTRUCT_ONLY |
                                 CLUTTER_PARAM_READWRITE);
@@ -987,7 +1009,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_boolean ("repeat-x",
                                 P_("Horizontal repeat"),
-                                P_("Repeat the contents rather than scaling them horizontally."),
+                                P_("Repeat the contents rather than scaling them horizontally"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   obj_props[PROP_REPEAT_X] = pspec;
@@ -995,7 +1017,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_boolean ("repeat-y",
                                 P_("Vertical repeat"),
-                                P_("Repeat the contents rather than scaling them vertically."),
+                                P_("Repeat the contents rather than scaling them vertically"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
   obj_props[PROP_REPEAT_Y] = pspec;
@@ -1003,7 +1025,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_enum ("filter-quality",
                              P_("Filter Quality"),
-                             P_("Rendering quality used when drawing the texture."),
+                             P_("Rendering quality used when drawing the texture"),
                              CLUTTER_TYPE_TEXTURE_QUALITY,
                              CLUTTER_TEXTURE_QUALITY_MEDIUM,
                              G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE);
@@ -1012,7 +1034,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
 
   pspec = g_param_spec_enum ("pixel-format",
                              P_("Pixel Format"),
-                             P_("The Cogl pixel format to use."),
+                             P_("The Cogl pixel format to use"),
                              COGL_TYPE_PIXEL_FORMAT,
                              COGL_PIXEL_FORMAT_RGBA_8888,
                              CLUTTER_PARAM_READABLE);
@@ -1020,16 +1042,16 @@ clutter_texture_class_init (ClutterTextureClass *klass)
   g_object_class_install_property (gobject_class, PROP_PIXEL_FORMAT, pspec);
 
   pspec = g_param_spec_boxed ("cogl-texture",
-                              P_("COGL Texture"),
-                              P_("The underlying COGL texture handle used to draw this actor"),
+                              P_("Cogl Texture"),
+                              P_("The underlying Cogl texture handle used to draw this actor"),
                               COGL_TYPE_HANDLE,
                               CLUTTER_PARAM_READWRITE);
   obj_props[PROP_COGL_TEXTURE] = pspec;
   g_object_class_install_property (gobject_class, PROP_COGL_TEXTURE, pspec);
 
   pspec = g_param_spec_boxed ("cogl-material",
-                              P_("COGL Material"),
-                              P_("The underlying COGL material handle used to draw this actor"),
+                              P_("Cogl Material"),
+                              P_("The underlying Cogl material handle used to draw this actor"),
                               COGL_TYPE_HANDLE,
                               CLUTTER_PARAM_READWRITE);
   obj_props[PROP_COGL_MATERIAL] = pspec;
@@ -1081,7 +1103,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
    */
   pspec = g_param_spec_boolean ("load-async",
                                 P_("Load asynchronously"),
-                                P_("Load files inside a thread to avoid blocking when loading images from disk."),
+                                P_("Load files inside a thread to avoid blocking when loading images from disk"),
                                 FALSE,
                                 CLUTTER_PARAM_WRITABLE);
   obj_props[PROP_LOAD_ASYNC] = pspec;
@@ -1098,7 +1120,7 @@ clutter_texture_class_init (ClutterTextureClass *klass)
    */
   pspec = g_param_spec_boolean ("load-data-async",
                                 P_("Load data asynchronously"),
-                                P_("Decode image data files inside a thread to reduce blocking when loading images from disk."),
+                                P_("Decode image data files inside a thread to reduce blocking when loading images from disk"),
                                 FALSE,
                                 CLUTTER_PARAM_WRITABLE);
   obj_props[PROP_LOAD_DATA_ASYNC] = pspec;
@@ -1206,7 +1228,10 @@ clutter_texture_set_custom_property (ClutterScriptable *scriptable,
 
       path = clutter_script_lookup_filename (script, str);
       if (G_UNLIKELY (!path))
-        return;
+        {
+          g_warning ("Unable to find image %s", str);
+          return;
+        }
 
       error = NULL;
       clutter_texture_set_from_file (texture, path, &error);
@@ -1518,7 +1543,7 @@ clutter_texture_set_from_data (ClutterTexture     *texture,
 
       g_set_error (&inner_error, CLUTTER_TEXTURE_ERROR,
                    CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
-                   "Failed to create COGL texture");
+                   "Failed to create Cogl texture");
 
       g_signal_emit (texture, texture_signals[LOAD_FINISHED], 0, inner_error);
 
@@ -1914,7 +1939,7 @@ clutter_texture_async_load (ClutterTexture *self,
     {
       g_set_error (error, CLUTTER_TEXTURE_ERROR,
 		   CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
-		   "Failed to create COGL texture");
+		   "Failed to create Cogl texture");
       return FALSE;
     }
   else
@@ -2009,7 +2034,7 @@ clutter_texture_set_from_file (ClutterTexture *texture,
     {
       g_set_error (&internal_error, CLUTTER_TEXTURE_ERROR,
                    CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
-		   "Failed to create COGL texture");
+		   "Failed to create Cogl texture");
     }
 
   if (internal_error != NULL)
@@ -2316,7 +2341,7 @@ clutter_texture_set_area_from_rgb_data (ClutterTexture     *texture,
     {
       g_set_error (error, CLUTTER_TEXTURE_ERROR,
 		   CLUTTER_TEXTURE_ERROR_BAD_FORMAT,
-		   "Failed to upload COGL texture data");
+		   "Failed to upload Cogl texture data");
       return FALSE;
     }
 
@@ -2338,8 +2363,27 @@ on_fbo_source_size_change (GObject          *object,
 {
   ClutterTexturePrivate *priv = texture->priv;
   gfloat w, h;
+  ClutterActorBox box;
+  gboolean status;
 
-  clutter_actor_get_transformed_size (priv->fbo_source, &w, &h);
+  status = clutter_actor_get_paint_box (priv->fbo_source, &box);
+  if (status)
+    clutter_actor_box_get_size (&box, &w, &h);
+
+  /* In the end we will size the framebuffer according to the paint
+   * box, but for code that does:
+   *   tex = clutter_texture_new_from_actor (src);
+   *   clutter_actor_get_size (tex, &width, &height);
+   * it seems more helpfull to return the src actor size if it has a
+   * degenerate paint box. The most likely reason it will have a
+   * degenerate paint box is simply that the src currently has no
+   * parent. */
+  if (status == FALSE || w == 0 || h == 0)
+    clutter_actor_get_size (priv->fbo_source, &w, &h);
+
+  /* We can't create a texture with a width or height of 0... */
+  w = MAX (1, w);
+  h = MAX (1, h);
 
   if (w != priv->image_width || h != priv->image_height)
     {
@@ -2396,6 +2440,21 @@ on_fbo_parent_change (ClutterActor        *actor,
       }
 }
 
+static void
+fbo_source_queue_redraw_cb (ClutterActor *source,
+			    ClutterActor *origin,
+			    ClutterTexture *texture)
+{
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (texture));
+}
+
+static void
+fbo_source_queue_relayout_cb (ClutterActor *source,
+			      ClutterTexture *texture)
+{
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (texture));
+}
+
 /**
  * clutter_texture_new_from_actor:
  * @actor: A source #ClutterActor
@@ -2459,8 +2518,8 @@ on_fbo_parent_change (ClutterActor        *actor,
  *   </listitem>
  *   <listitem>
  *     <para>Uploading pixel data to the texture (e.g by using
- *     clutter_actor_set_from_file()) will destroy the offscreen texture data
- *     and end redirection.</para>
+ *     clutter_texture_set_from_file()) will destroy the offscreen texture
+ *     data and end redirection.</para>
  *   </listitem>
  *   <listitem>
  *     <para>cogl_texture_get_data() with the handle returned by
@@ -2479,6 +2538,8 @@ clutter_texture_new_from_actor (ClutterActor *actor)
   ClutterTexture        *texture;
   ClutterTexturePrivate *priv;
   gfloat w, h;
+  ClutterActorBox box;
+  gboolean status;
 
   g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), NULL);
 
@@ -2493,10 +2554,25 @@ clutter_texture_new_from_actor (ClutterActor *actor)
 	return NULL;
     }
 
-  clutter_actor_get_transformed_size (actor, &w, &h);
+  status = clutter_actor_get_paint_box (actor, &box);
+  if (status)
+    clutter_actor_box_get_size (&box, &w, &h);
 
-  if (w == 0 || h == 0)
-    return NULL;
+  /* In the end we will size the framebuffer according to the paint
+   * box, but for code that does:
+   *   tex = clutter_texture_new_from_actor (src);
+   *   clutter_actor_get_size (tex, &width, &height);
+   * it seems more helpfull to return the src actor size if it has a
+   * degenerate paint box. The most likely reason it will have a
+   * degenerate paint box is simply that the src currently has no
+   * parent. */
+  if (status == FALSE || w == 0 || h == 0)
+    clutter_actor_get_size (actor, &w, &h);
+
+  /* We can't create a 0x0 fbo so always bump the size up to at least
+   * 1 */
+  w = MAX (1, w);
+  h = MAX (1, h);
 
   /* Hopefully now were good.. */
   texture = g_object_new (CLUTTER_TYPE_TEXTURE,
@@ -2542,6 +2618,11 @@ clutter_texture_new_from_actor (ClutterActor *actor)
                     G_CALLBACK(on_fbo_source_size_change),
                     texture);
 
+  g_signal_connect (actor, "queue-relayout",
+                    G_CALLBACK (fbo_source_queue_relayout_cb), texture);
+  g_signal_connect (actor, "queue-redraw",
+                    G_CALLBACK (fbo_source_queue_redraw_cb), texture);
+
   /* And a warning if the source becomes a child of the texture */
   g_signal_connect (actor,
                     "parent-set",
@@ -2585,6 +2666,16 @@ texture_fbo_free_resources (ClutterTexture *texture)
                              G_CALLBACK(on_fbo_source_size_change),
                              texture);
 
+      g_signal_handlers_disconnect_by_func
+                            (priv->fbo_source,
+                             G_CALLBACK(fbo_source_queue_relayout_cb),
+                             texture);
+
+      g_signal_handlers_disconnect_by_func
+                            (priv->fbo_source,
+                             G_CALLBACK(fbo_source_queue_redraw_cb),
+                             texture);
+
       g_object_unref (priv->fbo_source);
 
       priv->fbo_source = NULL;
